use std::collections::HashMap;
use std::sync::{Arc, Mutex};
use std::sync::mpsc::{channel, Sender, Receiver};

pub struct EventEmitter {
    /// 事件名称和对应的监听器列表
    _listeners: std::collections::HashMap<String, Vec<Box<dyn FnMut()>>>,
    /// 最大监听器数量
    _max_listeners: usize,
}

impl EventEmitter {
    /// 添加事件监听器
    pub fn on(&mut self, event_name: String, listener: Box<dyn FnMut()>) {
        // 如果事件名称不存在，则创建一个新的监听器列表
        if !self._listeners.contains_key(&event_name) {
            self._listeners.insert(event_name.clone(), Vec::new());
        }
        // 如果监听器数量已经达到最大值，则不再添加新的监听器
        if self._listeners[&event_name].len() >= self._max_listeners {
            return;
        }
        // 添加新的监听器
        self._listeners.get_mut(&event_name).unwrap().push(listener);
    }

    /// 移除事件监听器
    pub fn off(&mut self, event_name: String, listener: Box<dyn FnMut()>) {
        // 如果事件名称不存在，则直接返回
        if !self._listeners.contains_key(&event_name) {
            return;
        }
        // 移除指定的监听器
        let listeners = self._listeners.get_mut(&event_name).unwrap();
        let index = listeners.iter().position(|x| std::ptr::eq(x, &listener));
        if let Some(index) = index {
            listeners.remove(index);
        }
    }

    /// 触发事件
    pub fn emit(&mut self, event_name: String) {
        // 如果事件名称不存在，则直接返回
        if !self._listeners.contains_key(&event_name) {
            return;
        }
        // 依次调用所有监听器
        let listeners = self._listeners.get_mut(&event_name).unwrap();
        for listener in listeners.iter_mut() {
            listener();
        }
    }

    /// 添加事件监听器，与 on 方法功能相同
    pub fn add_listener(&mut self, event_name: String, listener: Box<dyn FnMut()>) {
        return self.on(event_name, listener);
    }

    /// 移除事件监听器，与 off 方法功能相同
    pub fn remove_listener(&mut self, event_name: String, listener: Box<dyn FnMut()>) {
        return self.off(event_name, listener);
    }

    /// 移除指定事件的所有监听器
    pub fn remove_all_listeners(&mut self, event_name: String) {
        self._listeners.remove(&event_name);
    }

    /// 添加一次性事件监听器
    // fn once(&mut self, event_name: String, listener: Box<dyn FnMut()>) {
    //     // 创建一个新的监听器，该监听器在执行后会自动移除
    //     let mut wrapper = Box::new(move || {
    //         listener();
    //         self.off(event_name.clone(), wrapper);
    //     });

    //     self.on(event_name, wrapper);
    // }

    /// 获取指定事件的监听器数量
    pub fn listener_count(&self, event_name: String) -> usize {
        return self._listeners.get(&event_name).map_or(0, |x| x.len())
    }

    /// 获取指定事件的监听器列表
    pub fn get_event_listeners(&self, event_name: String) -> Option<&Vec<Box<dyn FnMut()>>> {
        return self._listeners.get(&event_name)
    }

    /// 获取所有事件的监听器列表 
    // fn listeners(&self) -> Vec<(String, Vec<Box<dyn FnMut()>>)> {
    //     return self._listeners.iter().map(|(k, v)| (k.clone(), v.clone())).collect()
    // }

    /// 设置最大监听器数量
    pub fn set_max_listeners(&mut self, max_listeners: usize) {
        self._max_listeners = max_listeners;
    }

    /// 在事件监听器列表的开头添加一个监听器
    pub fn prepend_listener(&mut self, event_name: String, listener: Box<dyn FnMut()>) {
        // 如果事件名称不存在，则创建一个新的监听器列表
        if !self._listeners.contains_key(&event_name) {
            self._listeners.insert(event_name.clone(), Vec::new());
        }
        // 如果监听器数量已经达到最大值，则不再添加新的监听器
        if self._listeners[&event_name].len() >= self._max_listeners {
            return;
        }
        // 添加新的监听器
        self._listeners.get_mut(&event_name).unwrap().insert(0, listener);
    }

    // 在事件监听器列表的开头添加一个一次性监听器
    // fn prepend_once_listener(&mut self, event_name: String, listener: Box<dyn FnMut()>) {
    //     // 创建一个新的监听器，该监听器在执行后会自动移除
    //     let mut wrapper = Box::new(move || {
    //         listener();
    //         self.off(event_name.clone(), wrapper);
    //     });

    //     self.prepend_listener(event_name, wrapper);
    // }
}



